import re
import os
import sys
from os.path import isfile, join
from subprocess import PIPE, Popen


def get_linkage(path):
    if sys.platform.startswith('linux'):
        args = ['/usr/bin/ldd', path]
    elif sys.platform == 'darwin':
        args = ['/usr/bin/otool', '-L', path]
    elif sys.platform == 'win32':
        msvc_dir = r'C:\Program Files (x86)\Microsoft Visual Studio 9.0'
        os.environ['PATH'] = ';'.join([join(msvc_dir, r'Common7\IDE'),
                                       os.getenv('PATH')])
        args = [join(msvc_dir, r'VC\bin\dumpbin.exe'),
                '/dependents', path]
    else:
        raise OSError("OS not supported: %s" % sys.platform)

    p = Popen(args, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate()
    if stderr and str(stderr).strip():
        raise Exception("%r error: %s\n%s" % (args, stderr))
    return(str(stdout))


def has_mkl(path):
    assert path.endswith(('.so', '.pyd'))
    assert isfile(path)

    if sys.platform.startswith('linux'):
        rx = r'libmkl_\w+\.so'
    elif sys.platform == 'darwin':
        rx = r'libmkl_\w+\.dylib'
    elif sys.platform == 'win32':
        rx = r'mkl_\w+\.dll'
    else:
        raise OSError("OS not supported: %s" % sys.platform)

    pat = re.compile(rx)
    output = get_linkage(path)
    return bool(pat.search(output))


def on_build_machine():
    if os.getenv('USER') == 'ilan':
        print("*** we are on a build machine ***")
        return True
    return False


def test_modules(mkl_linked_modules):
    if not on_build_machine():
        return
    print("testing for MKL linkage")
    print('NOMKL=%r' % os.environ['NOMKL'])
    mkl = not int(os.getenv('NOMKL', 0))
    print("MKL: %s" % mkl)
    for m in mkl_linked_modules:
        path = getattr(m, '__file__')
        print(path)
        res = has_mkl(path)
        print('RESULT: %s' % res)
        assert mkl is res, path
