import re
import os
import sys
from os.path import join


def get_mkl_version_header():
    if sys.platform == 'win32':
        prefix = join(sys.prefix, 'Library')
    else:
        prefix = sys.prefix

    header_path = join(prefix, 'include', 'mkl_version.h')
    pat = re.compile(r'#define\s+__INTEL_MKL(\w*)__\s+(\d+)')
    parts = {}
    try:
        for line in open(header_path):
            m = pat.match(line.strip())
            if m:
                parts[m.group(1)] = m.group(2)
    except IOError:
        return '<NO MKL HEADER FOUND>'
    return '.'.join(parts[k] for k in ('', '_MINOR', '_UPDATE'))


def usage():
    sys.stdout.write("""usage: mkl-vesion FILE

read the MKL-version from <sys.prefix>/include/mkl_version.h, and replace
`__MKL_VERSION__` in FILE with the version.
Special options:

  --help
  --stdout       print the MKL version is printed to stdout
  --check        check numpy.__mkl_version__ (if present) with NOMKL
""")


def main():
    if len(sys.argv) != 2:
        usage()
    arg = sys.argv[1]

    if arg == '--stdout':
        print(get_mkl_version_header())
    elif arg == '--check':
        import numpy
        np_mkl = getattr(numpy, '__mkl_version__', None)
        env_mkl = not int(os.getenv('NOMKL', 0))
        print('NUMPY MKL: %r' % np_mkl)
        print('ENV MKL: %r' % env_mkl)
        assert bool(np_mkl) is env_mkl
    elif arg == '--help':
        usage()
    elif arg.startswith('--'):
        usage()
        sys.exit(1)
    else: # no -- option
        from cio_bt.replace import replace
        replace([('__MKL_VERSION__', get_mkl_version_header())], arg)


if __name__ == '__main__':
    main()
